#include "sdk.h"
#include "images.h"

typedef union {
	int rgba;
	struct {
		byte r;
		byte g;
		byte b;
		byte a;
	} c;
} color_t;

typedef struct {
	byte name[256];
} filename_t;

filename_t *files = 0x08804000;

void *fb = (void *)0x44000000;

int intro = 1;

unsigned int *sndfix = 0x08BF8A44;

#define MAXFILES 10
SceIoDirent file;
int numfiles;

byte *path = 0x08804000 + (MAXFILES * 256);

void SetColor(int col)
{
	int i;
	color_t *pixel = (color_t *)fb;
	for(i = 0; i < 512*272; i++) {
		pixel->rgba = col;
		pixel++;
	}
}

void DrawBack(int curcol)
{
	color_t col;
	col.rgba = 0x00FFFFFF;
	int i = curcol & 0xFF;
	switch(curcol & 0x0300) {
		case 0x0000:
			col.c.r = 255 - i;
			col.c.b = i;
		break;
		case 0x0100:
			col.c.g = 255 - i;
			col.c.r = i;
		break;
		case 0x0200:
			col.c.g = i;
			col.c.b = 255 - i;
		break;
	}
	SetColor(col.rgba);
}

void Swap()
{
	sceDisplaySetFrameBuf(fb, 512, PSP_DISPLAY_PIXEL_FORMAT_8888, 0);
	if(fb == 0x44000000) fb = (void *)0x44088000; else fb = (void *)0x44000000;
}

void PutImage(unsigned int *buff, int w, int h, int px, int py, int col)
{
	int x, y, i;
	color_t *pixel;
	for(y = 0; y < h; y++) {
		pixel = (color_t *)fb;
		pixel += (py + y) * 512 + px;
		for(x = 0; x < w; x++) {
			for(i = 0; i < 32; i++) {
				if((x * 32) + i + px > 479) continue;
				if(*buff & (1 << i)) {
					if(col) pixel->rgba = col; else {
						pixel->c.r = 255 - pixel->c.r;
						pixel->c.g = 255 - pixel->c.g;
						pixel->c.b = 255 - pixel->c.b;
					}
				}
				pixel++;
			}
			buff++;
		}
		if(y + py > 271) break;
	}
}

void PutChar(int in, int px, int py, int col)
{
	int x, y;
	color_t *pixel;

	byte *buff = &font_img;
	buff += 16 * in;

	for(y = 0; y < 16; y++) {
		pixel = (color_t *)fb;
		pixel += (py + y) * 512 + px;
		for(x = 0; x < 8; x++) {
			if(*buff & (128 >> x)) {
				if(col) pixel->rgba = col; else {
					pixel->c.r = 255 - pixel->c.r;
					pixel->c.g = 255 - pixel->c.g;
					pixel->c.b = 255 - pixel->c.b;
				}
			}
			pixel++;
		}
		buff++;
	}
}

void write(byte *text, int px, int py, int col)
{
	int ox = px;
	py += 2; // new font hack-patch
	while(*text) {
		if(*text == '\n') {
			py += 16;
			px = ox;
			text++;
			continue;
		}
		if(*text == '\t') {
			px += 9 * 2;
			text++;
			continue;
		}
		if(*text > 0x20 && *text < 0x7E) PutChar(*text - 0x21, px, py, col);
		px += 9;
		text++;
	}
}

int ScanDir()
{
	int i;
	SceUID uid;

	numfiles = 0;
	uid = sceIoDopen("ms0:/bin");
	if(uid >= 0) {
		while(numfiles < MAXFILES) {
			i = sceIoDread(uid, &file);
			if(i <= 0) break;
			if(file.d_stat.st_mode == 4607) continue;
			memcpy(&files[numfiles].name, &file.d_name, 256);
			numfiles++;
		}
		sceIoDclose(uid);
	} else return uid;
	return 0;
}

byte hextab[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

void HEX(byte *buff, unsigned int num)
{
	buff[0] = '0';
	buff[1] = 'x';
	buff[2] = hextab[(num & 0xF0000000) >> 28];
	buff[3] = hextab[(num & 0x0F000000) >> 24];
	buff[4] = hextab[(num & 0x00F00000) >> 20];
	buff[5] = hextab[(num & 0x000F0000) >> 16];
	buff[6] = hextab[(num & 0x0000F000) >> 12];
	buff[7] = hextab[(num & 0x00000F00) >> 8];
	buff[8] = hextab[(num & 0x000000F0) >> 4];
	buff[9] = hextab[num & 0x0000000F];
	buff[10] = 0;
}

void _start(unsigned long, unsigned long *) __attribute__ ((section (".text.start")));
void _start(unsigned long arglen, unsigned long *argp)
{
	// disable MOHH ambience
	sndfix[0] = 0x0E324AA5;
	sndfix[1] = 0x00000000;

	sceDisplaySetFrameBuf(fb, 512, PSP_DISPLAY_PIXEL_FORMAT_8888, 1);

	int i;
	int curcol = 0;
	color_t col;
	int menu[4] = {0x00FFFFFF, 0xFF000000, 0xFF000000, 0xFF000000};
	int pos = 0;
	int bt = 0;
	int state = 0;
	int cf = 0;
	SceCtrlData pad;
	SceUID uid;

	memset(files, 0, 256 * MAXFILES);
	memset(&file, 0, sizeof(SceIoDirent));

	uid = sceIoOpen("ms0:/hx.conf", PSP_O_RDONLY, 777);
	if(uid >= 0) {
		sceIoRead(uid, &intro, 4);
		sceIoClose(uid);
	}

	if(intro) {
		SetColor(0x00FFFFFF);
		Swap();
		sceKernelDelayThread(500*1000);

		for(i = 255; i >= 0; i--) {
			col.c.r = i;
			col.c.g = i;
			col.c.b = i;
			SetColor(col.rgba);
			PutImage(&kgsws_img, kgsws_w, kgsws_h, 160, 112, 0x00FFFFFF);
			Swap();
			sceKernelDelayThread(4*1000);
		}

		for(i = 112; i > 4; i--) {
			SetColor(0);
			PutImage(&kgsws_img, kgsws_w, kgsws_h, 160, i, 0x00FFFFFF);
			Swap();
			sceKernelDelayThread(1000);
		}

		for(i = 1; i < 256; i++) {
			col.c.r = i;
			col.c.g = i;
			col.c.b = i;
			SetColor(0);
			PutImage(&kgsws_img, kgsws_w, kgsws_h, 160, 5, 0x00FFFFFF);
			PutImage(&presents_img, presents_w, presents_h, 144, 119, col.rgba);
			Swap();
			sceKernelDelayThread(4 * 1000);
		}

		for(i = 144; i < 480; i++) {
			SetColor(0);
			PutImage(&kgsws_img, kgsws_w, kgsws_h, 160, 5, 0x00FFFFFF);
			PutImage(&presents_img, presents_w, presents_h, i, 119, 0x00FFFFFF);
			Swap();
			sceKernelDelayThread(200);
		}

		SetColor(0x00FFFFFF);
		Swap();
		sceKernelDelayThread(30 * 1000);
		SetColor(0);
		PutImage(&kgsws_img, kgsws_w, kgsws_h, 160, 5, 0x00FFFFFF);
		PutImage(&mohh_img, mohh_w, mohh_h, 120, 60, 0x00FFFFFF);
		Swap();
		sceKernelDelayThread(500 * 1000);
		SetColor(0x00FFFFFF);
		Swap();
		sceKernelDelayThread(30 * 1000);
		SetColor(0);
		PutImage(&kgsws_img, kgsws_w, kgsws_h, 160, 5, 0x00FFFFFF);
		PutImage(&mohh_img, mohh_w, mohh_h, 120, 60, 0x00FFFFFF);
		PutImage(&exploit_img, exploit_w, exploit_h, 265, 60, 0x00FFFFFF);
		Swap();

		sceKernelDelayThread(200 * 1000);

		for(i = 1; i < 255; i++) {
			col.c.r = i;
			col.c.g = i;
			col.c.b = 0;
			SetColor(col.rgba);
			PutImage(&kgsws_img, kgsws_w, kgsws_h, 160, 5, 0);
			PutImage(&ss_img, ss_w, ss_h, 324, 7, 0);
			PutImage(&mohh_img, mohh_w, mohh_h, 120, 60, 0);
			PutImage(&exploit_img, exploit_w, exploit_h, 265, 60, 0);
			PutImage(&m1_img, m1_w, m1_h, 200, 120, 0xFF000000);
			PutImage(&m2_img, m2_w, m2_h, 200, 140, 0xFF000000);
			PutImage(&m3_img, m3_w, m3_h, 200, 160, 0xFF000000);
			PutImage(&m4_img, m4_w, m4_h, 200, 180, 0xFF000000);
			write("enabled", 256, 120, 0xFF000000);
			col.c.r = 0;
			col.c.g = 0;
			col.c.b = i;
			PutImage(&ss_img, ss_w, ss_h, 324, 7, col.rgba);
			write("This is second version :)", 10, 251, col.rgba);
			Swap();
			sceKernelDelayThread(3 * 1000);
		}
	}

	while(1) {
		if(bt) bt--; else {
			sceCtrlPeekBufferPositive(&pad, 1);
			if(pad.Buttons & PSP_CTRL_CROSS) {
				if(!state) {
					if(pos == 0) {
						intro ^= 1;
						uid = sceIoOpen("ms0:/hx.conf", PSP_O_WRONLY | PSP_O_TRUNC | PSP_O_CREAT, 777);
						if(uid >= 0) {
							sceIoWrite(uid, &intro, 4);
							sceIoClose(uid);
						}
					}
					if(pos == 1) {
						cf = 0;
						state = 2;
						if(ScanDir() < 0) state = 4; else if(numfiles == 0) state = 5;
					}
					if(pos == 2) state = 3;
					if(pos == 3) break;
				} else if(state == 2) {
					memcpy(path, "ms0:/bin/", 9);
					memcpy(path + 9, files[cf].name, 256);
					uid = sceIoOpen(path, PSP_O_RDONLY, 777);
					if(uid >= 0) {
						fb = (unsigned int *)0x44000000;
						SetColor(0);
						sceDisplaySetFrameBuf(0x04000000, 512, PSP_DISPLAY_PIXEL_FORMAT_8888, 0);
						sceIoRead(uid, 0x08E50000, 0x01000000);
						sceIoClose(uid);
						sceKernelDcacheWritebackInvalidateAll();
						sceKernelDelayThread(1000*200);
						asm("nop");
						asm("j 0x08E50000");
						asm("nop");
					}
					HEX(path, uid);
					state = 6;
				}
				pad.Buttons = 0;
				bt = 25;
			}
			if(pad.Buttons & PSP_CTRL_CIRCLE) {
				state = 0;
				pad.Buttons = 0;
			}
			if(pad.Buttons & (PSP_CTRL_UP | PSP_CTRL_DOWN)) {
				if(!state) {
					menu[0] = 0xFF000000;
					menu[1] = 0xFF000000;
					menu[2] = 0xFF000000;
					menu[3] = 0xFF000000;
					if(pad.Buttons & PSP_CTRL_UP) pos--;
					if(pad.Buttons & PSP_CTRL_DOWN) pos++;
					if(pos > 3) pos = 0;
					if(pos < 0) pos = 3;
					menu[pos] = 0x00FFFFFF;
				}
				if(state == 2) {
					if(pad.Buttons & PSP_CTRL_UP) cf--;
					if(pad.Buttons & PSP_CTRL_DOWN) cf++;
					if(cf == numfiles) cf = 0;
					if(cf < 0) cf = numfiles - 1;
				}
				bt = 15;
			}
		}
		curcol++;
		if(curcol == 0x0300) curcol = 0;
		DrawBack(curcol);
		PutImage(&kgsws_img, kgsws_w, kgsws_h, 160, 5, 0);
		PutImage(&ss_img, ss_w, ss_h, 324, 7, 0);
		PutImage(&mohh_img, mohh_w, mohh_h, 120, 60, 0);
		PutImage(&exploit_img, exploit_w, exploit_h, 265, 60, 0);
		switch(state) {
			case 0:
				PutImage(&m1_img, m1_w, m1_h, 200, 120, menu[0]);
				PutImage(&m2_img, m2_w, m2_h, 200, 140, menu[1]);
				PutImage(&m3_img, m3_w, m3_h, 200, 160, menu[2]);
				PutImage(&m4_img, m4_w, m4_h, 200, 180, menu[3]);
				if(intro) write("enabled", 256, 120, menu[0]); else write("disabled", 256, 120, menu[0]);
			break;
			case 2:
				for(i = 0; i < numfiles; i++) {
					if(i == cf) col.rgba = 0x00FFFFFF; else col.rgba = 0xFF000000;
					write(files[i].name, 240 - (strlen(files[i].name) * 9) / 2 , 90 + 16 * i, col.rgba);
				}
			break;
			case 3:
				write("EA", 120, 90, 0x00FFFFFF);
				write("MOHH game is good, but it\ncan be better if you fix\nonline servers.\n\n- death players should not be able to shoot\n- player speed can be checked\n- same for shoot speed\n- if someone is kicked, disconnect him\n- player should be able to delete only his personas", 10, 106, 0xFF000000);

				write("Sony", 330, 90, 0x00FFFFFF);
				write("PSP is good console,\nbut we can't create\nour own programs\nto it ...", 280, 106, 0xFF000000);
			break;
			case 4:
				write("Does folder ms0:/bin exist?", 130, 140, 0xFF000000);
			break;
			case 5:
				write("Is'nt folder ms0:/bin empty?", 130, 140, 0xFF000000);
			break;
			case 6:
				write("Error loading file:", 170, 140, 0xFF000000);
				write(path, 200, 156, 0xFF000000);
			break;
		}
		write("This is second version :)", 10, 251, 0);
		Swap();
		sceKernelDelayThread(3*1000);
	}
	SetColor(0x00FFFFFF);
	Swap();
	sceKernelExitGame();
	return 0;
}
